/// @file   NsOracle.h
/// @brief  Oracle NS Implementation.
/// @author Alejandro Álvarez Ayllón <aalvarez@cern.ch>
#ifndef NSORACLEL_H
#define	NSORACLE_H

#include <dirent.h>
#include <dmlite/cpp/inode.h>
#include <occi.h>
#include <vector>

namespace dmlite {

/// Struct used internally to read directories.
  struct NsOracleDir: public IDirectory {
    virtual ~NsOracleDir() {};
    
    uint64_t                 dirId;         ///< The directory ID.
    ExtendedStat             current;       ///< Current entry metadata.
    struct dirent            ds;            ///< The structure used to hold the returned data.
    oracle::occi::Statement *stmt;          ///< The statement.
    oracle::occi::ResultSet *rs;            ///< The result set.
  };

  /// Implementation of NS Oracle backend.
  class INodeOracle: public INode {
   public:

    /// Constructor
    /// @param pool     The Oracle connection pool.
    /// @param conn     The Oracle connection.
    INodeOracle(oracle::occi::ConnectionPool* pool,
                oracle::occi::Connection* conn) throw (DmException);

    /// Destructor
    ~INodeOracle() throw (DmException);

    // Overloading
    std::string getImplId(void) const throw ();

    void setStackInstance(StackInstance* si) throw (DmException);
    void setSecurityContext(const SecurityContext* ctx) throw (DmException);

    void begin(void) throw (DmException);
    void commit(void) throw (DmException);
    void rollback(void) throw (DmException);

    ExtendedStat create(const ExtendedStat&) throw (DmException);

    void symlink(ino_t inode, const std::string &link) throw (DmException);

    void unlink(ino_t inode) throw (DmException);

    void move  (ino_t inode, ino_t dest) throw (DmException);
    void rename(ino_t inode, const std::string& name) throw (DmException);

    ExtendedStat extendedStat(ino_t inode) throw (DmException);
    ExtendedStat extendedStat(ino_t parent, const std::string& name) throw (DmException);
    ExtendedStat extendedStat(const std::string& guid) throw (DmException);

    SymLink readLink(ino_t inode) throw (DmException);

    void addReplica   (const Replica& replica) throw (DmException);  
    void deleteReplica(const Replica& replica) throw (DmException);

    std::vector<Replica> getReplicas(ino_t inode) throw (DmException);

    Replica getReplica   (int64_t rid) throw (DmException);
    Replica getReplica   (const std::string& sfn) throw (DmException);
    void    updateReplica(const Replica& replica) throw (DmException);

    void utime(ino_t inode, const struct utimbuf* buf) throw (DmException);

    void setMode (ino_t inode, uid_t uid, gid_t gid,
                  mode_t mode, const Acl& acl) throw (DmException);

    void setSize    (ino_t inode, size_t size) throw (DmException);
    void setChecksum(ino_t inode, const std::string& csumtype,
                    const std::string& csumvalue) throw (DmException);

    std::string getComment   (ino_t inode) throw (DmException);
    void        setComment   (ino_t inode, const std::string& comment) throw (DmException);
    void        deleteComment(ino_t inode) throw (DmException);

    void setGuid(ino_t inode, const std::string& guid) throw (DmException);
    
    void updateExtendedAttributes(ino_t inode,
                                  const Extensible& attr) throw (DmException);

    IDirectory*    openDir (ino_t inode) throw (DmException);
    void           closeDir(IDirectory* dir) throw (DmException);  
    ExtendedStat*  readDirx(IDirectory* dir) throw (DmException);
    struct dirent* readDir (IDirectory* dir) throw (DmException);

   protected:
    /// The Oracle connection pool.
    oracle::occi::ConnectionPool* pool_;

    /// The Oracle connection
    oracle::occi::Connection* conn_;

    /// Transaction level, so begins and commits can be nested.
    unsigned transactionLevel_;

   private:
    /// Update nlinks
    void updateNlink(ino_t inode, int diff) throw (DmException);
  };

};

#endif	// NSORACLE_H
